/*
 * Copyright (c) 2006, John Mettraux, OpenWFE.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * . Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.  
 * 
 * . Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 * 
 * . Neither the name of the "OpenWFE" nor the names of its contributors may be
 *   used to endorse or promote products derived from this software without
 *   specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: AbstractService.java 2587 2006-05-09 09:15:59Z jmettraux $
 */

//
// DecisionParticipant.java
//
// john.mettraux@openwfe.org
//
// generated with 
// jtmpl 1.1.01 2004/05/19 (john.mettraux@openwfe.org)
//

package openwfe.org.decision.impl;

import openwfe.org.MapUtils;
import openwfe.org.ApplicationContext;
import openwfe.org.engine.Definitions;
import openwfe.org.engine.workitem.WorkItem;
import openwfe.org.engine.workitem.InFlowWorkItem;
import openwfe.org.engine.dispatch.DispatchingException;
import openwfe.org.engine.expressions.FlowExpression;
import openwfe.org.engine.participants.LeafParticipant;
import openwfe.org.decision.DecisionService;
import openwfe.org.decision.DecisionException;


/**
 * A participant that applies a decision table.
 *
 * <p><font size=2>CVS Info :
 * <br>$Author$
 * <br>$Id$ </font>
 *
 * @author john.mettraux@openwfe.org
 */
public class DecisionParticipant

    extends LeafParticipant

{

    private final static org.apache.log4j.Logger log = org.apache.log4j.Logger
        .getLogger(DecisionParticipant.class.getName());

    //
    // CONSTANTS & co

    /**
     * The parameter 'tableName' points to the name of a decision table
     * within the local decisionService.
     */
    public final static String P_TABLE_NAME
        = "tableName";

    /**
     * By default, the 'decisionService' parameter points to 'decisionService'.
     */
    public final static String P_DECISION_SERVICE
        = "decisionService";

    //
    // FIELDS

    private String tableName = null;
    private String decisionService = null;

    //
    // CONSTRUCTORS

    public void init
        (final String regex,
         final java.util.Map params)
    {
        super.init(regex, params);

        this.tableName = MapUtils
            .getMandatoryString(params, P_TABLE_NAME);

        this.decisionService = MapUtils
            .getAsString(params, P_DECISION_SERVICE, P_DECISION_SERVICE);

        log.info("init() decisionService '"+decisionService+"'");
        log.info("init() tableName       '"+tableName+"'");
    }

    //
    // METHODS from Participant

    public Object dispatch
        (final ApplicationContext context, final WorkItem wi)
    throws
        DispatchingException
    {
        final InFlowWorkItem ifwi = (InFlowWorkItem)wi;

        final FlowExpression fe = Definitions
            .getExpressionPool(context).fetch(ifwi.getLastExpressionId());

        final DecisionService ds = 
            (DecisionService)context.get(this.decisionService);

        try
        {
            ds.apply(tableName, fe, ifwi);
        }
        catch (final DecisionException e)
        {
            throw new DispatchingException
                ("failed to apply the workitem in the decision table '"+
                 tableName+"'", e);
        }

        log.debug("dispatch() decision application done");

        //
        // reply

        reply(context, ifwi);

        return null;
            //
            // no feedback required
    }

    //
    // METHODS

    //
    // STATIC METHODS

}
