/*
 * Copyright (c) 2006, John Mettraux, OpenWFE.org
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * . Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.  
 * 
 * . Redistributions in binary form must reproduce the above copyright notice, 
 *   this list of conditions and the following disclaimer in the documentation 
 *   and/or other materials provided with the distribution.
 * 
 * . Neither the name of the "OpenWFE" nor the names of its contributors may be
 *   used to endorse or promote products derived from this software without
 *   specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: AbstractService.java 2587 2006-05-09 09:15:59Z jmettraux $
 */

//
// SimpleDecisionTable.java
//
// john.mettraux@openwfe.org
//
// generated with 
// jtmpl 1.1.01 2004/05/19 (john.mettraux@openwfe.org)
//

package openwfe.org.decision.impl;

import openwfe.org.Utils;
import openwfe.org.misc.Text;
import openwfe.org.engine.workitem.Attribute;
import openwfe.org.engine.workitem.CollectionAttribute;
import openwfe.org.engine.workitem.InFlowWorkItem;
import openwfe.org.engine.expressions.VariableMap;
import openwfe.org.engine.expressions.FlowExpression;
import openwfe.org.decision.DecisionTable;


/**
 * The default implementation of a DecisionTable.
 *
 * <p><font size=2>CVS Info :
 * <br>$Author$
 * <br>$Id$ </font>
 *
 * @author john.mettraux@openwfe.org
 */
public class SimpleDecisionTable

    implements DecisionTable

{

    private final static org.apache.log4j.Logger log = org.apache.log4j.Logger
        .getLogger(SimpleDecisionTable.class.getName());

    //
    // CONSTANTS & co

    //
    // FIELDS

    private String[] ins = null;
    private String[] outs = null;
    private java.util.List rows = null;

    private boolean firstMatch = true;

    //
    // CONSTRUCTORS

    //
    // METHODS

    /**
     * Sets the list of field names that are taken as input of a rule 
     * evaluation.
     */
    public void setIns (final String[] ss)
    {
        this.ins = ss;
    }

    public void setIns (final java.util.List l)
    {
        this.ins = Utils.toStringArray(l);
    }

    /**
     * Sets the list of field names that may be impacted 
     * by a rule.
     */
    public void setOuts (final String[] ss)
    {
        this.outs = ss;
    }

    public void setOuts (final java.util.List l)
    {
        this.outs = Utils.toStringArray(l);
    }

    /**
     * The list of lines (lists too) with the values that define
     * the rules.
     */
    public void setRows (final java.util.List l)
    {
        this.rows = l;
    }

    /**
     * If 'firstMatch' is set to true, the rule evaluation will stop
     * as soon as one rule fully matches. Else all the rules will be
     * evaluated, seeing their results (outs) cumulated.
     */
    public void setFirstMatch (final boolean b)
    {
        this.firstMatch = b;
    }

    //
    // METHODS from DecisionTable

    /* *
     * Applies the rules in the decision table to the given map of fields.
     * /
    public void apply (final java.util.Map m)
    {
        final StringMapAttribute sma = AttributeUtils.map2owfe(m);

        apply(sma);

        return AttributeUtils.map2java(sma);
    }
    */

    /**
     * Applies the rules in the decision table to the given map of fields.
     */
    public void apply 
        (final InFlowWorkItem wi)
    {
        apply(null, wi);
    }

    /**
     * Applies the rules in the decision table to the given map of fields.
     */
    public void apply 
        (final FlowExpression fe, final InFlowWorkItem wi)
    {
        final java.util.Iterator rit = this.rows.iterator();
        while (rit.hasNext())
        {
            final String[][] sss = (String[][])rit.next();

            final String[] rins = sss[0];
            final String[] routs = sss[1];

            boolean valid = true;

            for (int i=0; i<ins.length; i++)
            {
                String fieldName = ins[i].trim();
                final String regex = rins[i].trim();

                fieldName = VariableMap.substitute(fe, wi, fieldName);

                final Attribute aValue = CollectionAttribute
                    .lookupAttribute(fieldName, wi.getAttributes());

                final String value = aValue.toString();

                if (log.isDebugEnabled())
                {
                    log.debug("apply() fieldName >"+fieldName+"<");
                    log.debug("apply() regex     >"+regex+"<");
                    log.debug("apply() value     >"+value+"<");
                }

                if (regex.length() > 0)
                {
                    if (value == null || ( ! value.matches(regex)))
                    {
                        valid = false;
                        break;
                    }
                }
            }

            if (valid) insertOuts(fe, wi, routs);

            if (valid && firstMatch)
            {
                log.debug("apply() found first match");
                break;
            }
        }

        if (log.isDebugEnabled()) log.debug("apply() over.");
    }

    /**
     * This method is called upon a successful rule evaluation, it inserts
     * the values specified within to the 'outs' fields.
     * The dollar notation is active :-)
     */
    protected void insertOuts
        (final FlowExpression fe, 
         final InFlowWorkItem wi,
         final String[] routs)
    {
        for (int i=0; i<outs.length; i++)
        {
            String fieldName = outs[i];
            String value = routs[i];

            if (log.isDebugEnabled())
            {
                log.debug("insertOuts() -1 fieldName  >"+fieldName+"<");
                log.debug("insertOuts() -1 value      >"+value+"<");
            }

            fieldName = VariableMap.substitute(fe, wi, fieldName);
            value = VariableMap.substitute(fe, wi, value);

            if (value.length() < 1) continue;

            if (log.isDebugEnabled())
            {
                log.debug("insertOuts()  0 fieldName  >"+fieldName+"<");
                log.debug("insertOuts()  0 value      >"+value+"<");
            }

            wi.getAttributes().setField(fieldName, value);
        }
    }

    //
    // STATIC METHODS

    //
    // INNER CLASSES

}
